# coffeelint: disable=max_line_length

_ = require "lodash"
fs = require 'fs-extra'
async = require 'async'
chalk = require 'chalk'

Producer = require '../../ProducerClass'

running = true
watcher = false

debug = require("../../../shared_modules/logger").debug

readFileIfExists = (path, cb) ->
  fs.exists path, (itExists) ->
    if itExists
      fs.readFile path, (err, contents) ->
        contents = contents.toString()
        cb err, contents
    else
      cb null, null

readJsonIfExists = (path, cb) ->
  fs.exists path, (itExists) ->
    if itExists
      debug "reading #{path}"
      fs.readJson path, (err, contents) ->
        cb null, contents
    else
      debug "#{path} does not exist"
      cb null, null


logToJson = (path, cb) ->
  readFileIfExists path, (err, log) ->
    lines = []
    if log
      lines = log.split /[\n\r]+/
      lines = (line.replace(/\'/g, '"') for line in lines when line)
    cb err, lines

getChannelLogs = (chan, cb) ->
  async.parallel {
    inbound: (callback) ->
      debug "getting inbound calls for #{chan.channelIndex}"
      logToJson "/tmp/inbound_call_#{chan.channelIndex}.log", callback
    outbound: (callback) ->
      logToJson "/tmp/outbound_call_#{chan.channelIndex}.log", callback
    ISDN: (callback) ->
      readJsonIfExists "/tmp/isdn-call-#{chan.channelIndex}.txt", callback

    channel: (callback) ->
      callback null, chan.channelIndex
    name: (callback) ->
      callback null, chan.name
  }, cb

class Callinfo extends Producer
  constructor: ->
    super "callinfo", 15 * 1000

    @start = =>
      debug "starting"
      @debug = debug
      watchDir = "/tmp/"

      ###
      checks if changed file was BChannelState, then runs getChannelLogs and sends result through zmq to prod manager
      ###
      # changedFile = ko.observable("").extend({rateLimit:2000, notify: "always"})

      # ko.computed =>
      getLogs = =>

        @requestData { producer: "channels", action: "getStatus" }, (err, status, data) =>
          if err
            debug "could not get channels, retry in 5 seconds", err
            setTimeout getLogs, 5000
            return

          debug "channels responded"
          # convert dchannels to array
          dchannels = (val for key, val of data)
          dchannels = _.sortBy dchannels, "channelIndex"

          debug "getting logs for all channels"
          async.map dchannels, getChannelLogs, (err, allLogs) =>
            debug chalk.red "error", err if err
            debug "got #{allLogs?.length} call logs"
            returnObject = {}
            _.each allLogs, (log) ->
              returnObject[log.name] = log
            @sendData returnObject

      fs.watch watchDir, _.throttle (event, filename) ->
        return unless running
        if filename.lastIndexOf("BChannelState", 0) is 0 or filename.lastIndexOf("isdn-call-", 0) is 0 or filename.lastIndexOf("inbound_call", 0) is 0 or filename.lastIndexOf("outbound_call", 0) is 0
          debug "a file changed", filename
          getLogs()
      , 2000

      getLogs()

    #start immediately
    @start()

    @stop = ->
      # watcher.close();
      running = false

  subscribe: =>
    if not running
      running = true
      @start()
    super()

module.exports = new Callinfo()
