
# coffeelint: disable=max_line_length

_ = require("lodash")
ko = require "knockout"
Producer = require "../../ProducerClass"
debug = require("../../../shared_modules/logger").debug

bchannelsLib = require "./bchannels"
dchannelsLib = require "./dchannels"

class Channels extends Producer
  constructor: ->
    super "channels", 10 * 1000

    @running = true

    dchannels = ko.observableArray()
    dchannelsLib.on "new", (channels) ->
      dchannels channels

    bchannels = ko.observableArray()
    bchannelsLib.on "new", (channels) ->
      bchannels channels

    channels = ko.computed ->
      # unless @running
        # debug "Not updating channels, noone called start"
        # return
      # only run when all channels are set
      unless dchannels().length and bchannels().length
        debug "not merging channel data because dchannels or bchannels are not set, yet"
        debug "bchannels().length #{bchannels().length} dchannels().length #{dchannels().length}"
        return
      debug "Merging channel data"
      mergedChannels = {}

      bchannelsGrouped = _.groupBy bchannels(), (bchan) ->
        bchanIlid = bchan.bchannel.split("/")[0]
        return bchanIlid

      _.each dchannels(), (dchan) ->
        dchanIlid = "#{dchan.ilidNumber}.#{dchan.ilidIndex}"

        #add matching bchannels to dchannel
        dchan.bchannels = bchannelsGrouped[dchanIlid]

        mergedChannels[dchan.name] = dchan

      return mergedChannels

    # send changes
    ko.computed =>
      @sendData ko.toJS channels
    .extend({ rateLimit: 100 })

    @subscribeTo "configurator", (err, status, data) ->
      if data and data.config and data.config.struct_Box and data.config.struct_Box.struct_DChannel
        # debug "from configurator: ", data.config.struct_Box.struct_DChannel
        dchannelsLib.getNewConfig()

    # supportedChannels = ko.computed =>
    #   dchannelsLocal = dchannels()
    #   dchannelsGrouped = _.groupBy dchannelsLocal, (chan) ->
    #     return chan.ilidType
    #   filteredChannels = {}
    #   _.each dchannelsGrouped, (chanObject, chanType) ->
    #     filteredChannels[chanType] = _.size chanObject
    #   return filteredChannels
    # .extend({notify: "always"})


  subscribe: =>
    super()
    debug "got subscription"
    if @running
      debug "already running"
      return
    debug "starting"
    @running = true
    dchannelsLib.start()
    bchannelsLib.start()

  stop: =>
    unless @running
      debug "Not running"
      return
    debug "stopping"
    @running = false
    dchannelsLib.stop()
    bchannelsLib.stop()


module.exports = Channels

return


# class Channels extends Producer
#   constructor: ->
#     super "channels", 10000

#     amountOfDifferentChannels = ko.computed ->
#       dchannels = dchannelsExtended()
#       dchannelsGrouped = _.groupBy dchannels, (chan) ->
#         return chan.ilidType
#       supportedChannels = {}
#       _.each dchannelsGrouped, (chanObject, chanType) ->
#         supportedChannels[chanType] = _.size chanObject
#       return supportedChannels

#     allChannels = ko.computed ->
#       bchannels: bchannels
#       dchannels: dchannelsExtended()
#       supportedChannels: amountOfDifferentChannels()


#     sendChanges = =>
#       activeChannels = _.filter bchannels, {active: "true"}
#       _.each activeChannels, (chan) ->
#         chan.runTime = calculateRuntime(chan.date)

#       @sendData ko.toJS allChannels()

#     allChannels.subscribe ->
#       sendChanges()
#     , true

#     @start = =>
#       # @sendData ko.toJS allChannels, true
#       clearInterval checkInterval if checkInterval
#       checkInterval = setInterval(sendChanges, interValTimeToGetNewData)

#       watcher = fs.watch(watchDir, addChangedFile) unless watcher

#       checkInitialBchannels = (err, files) ->
#         _.each files, (file) ->
#           addChangedFile null, file
#       fs.readdir(watchDir, checkInitialBchannels)

#       debug "started"

#     @stop = =>
#       watcher.close() if watcher
#       watcher = false
#       clearInterval checkInterval
#       isWatching = false
#       debug "stopped, isWatching is #{isWatching}"

#   subscribe: ->
#     unless isWatching
#       debug "Not watching, subscription possible"
#       isWatching = true
#       @start()
#     super


# module.exports = new Channels()
